@@ -24,8 +24,6 @@ class PAOFLOW:
 
   comm = rank = size = None
 
-  workpath = inputfile = None
-
   start_time = reset_time = None
 
   # Overestimate factor for guessing memory requirements
@@ -52,7 +50,7 @@ class PAOFLOW:
 
 
 
-  def __init__ ( self, workpath='./', outputdir='output', inputfile=None, savedir=None, smearing='gauss', npool=1, verbose=False ):
+  def __init__ ( self, workpath='./', outputdir='output', inputfile=None, savedir=None, npool=1, smearing='gauss', non_ortho=False, verbose=False, restart=False ):
     '''
     Initialize the PAOFLOW class, either with a save directory with required QE output or with an xml inputfile
 
@@ -61,9 +59,11 @@ class PAOFLOW:
         outputdir (str): Name of the output directory (created in the working directory path)
         inputfile (str): (optional) Name of the xml inputfile
         savedir (str): QE .save directory
-        smearing (str): Smearing type (None, m-p, gauss)
         npool (int): The number of pools to use. Increasing npool may reduce memory requirements.
+        smearing (str): Smearing type (None, m-p, gauss)
+        non_ortho (bool): If True the Hamiltonian will be Orthogonalized after construction
         verbose (bool): False supresses debugging output
+        restart (bool): True if the run is being restarted from a .json data dump.
 
     Returns:
         None
@@ -73,10 +73,6 @@ class PAOFLOW:
     from .defs.header import header
     from .DataController import DataController
 
-    self.workpath = workpath
-    self.outputdir = outputdir
-    self.inputfile = inputfile
-
     #-------------------------------
     # Initialize Parallel Execution
     #-------------------------------
@@ -93,33 +89,38 @@ class PAOFLOW:
       self.start_time = self.reset_time = time()
 
     # Initialize Data Controller
-    self.data_controller = DataController(workpath, outputdir, inputfile, savedir, smearing, npool, verbose)
+    self.data_controller = DataController(workpath, outputdir, inputfile, savedir, npool, smearing, non_ortho, verbose, restart)
 
-    # Data Attributes
-    attr = self.data_controller.data_attributes
     self.report_exception = self.data_controller.report_exception
 
-    # Check for CUDA FFT Libraries
-## CUDA not yet supported in PAOFLOW_CLASS
-    attr['use_cuda'] = False
-    attr['scipyfft'] = True
-    if attr['use_cuda']:
-      attr['scipyfft'] = False
-    if self.rank == 0 and attr['verbose']:
+    if not restart:
+      # Data Attributes
+      attr = self.data_controller.data_attributes
+
+      # Check for CUDA FFT Libraries
+      ## CUDA not yet supported in PAOFLOW_CLASS
+      attr['use_cuda'] = False
+      attr['scipyfft'] = True
       if attr['use_cuda']:
-        print('CUDA will perform FFTs on %d GPUs'%1)
-      else:
-        print('SciPy will perform FFTs')
+        attr['scipyfft'] = False
+      if self.rank == 0 and attr['verbose']:
+        if attr['use_cuda']:
+          print('CUDA will perform FFTs on %d GPUs'%1)
+        else:
+          print('SciPy will perform FFTs')
 
     # Report execution information
     if self.rank == 0:
-      if self.size == 1:
-        print('Serial execution')
+      if restart:
+        print('Run starting from Restart data.')
       else:
-        print('Parallel execution on %d processors and %d pool'%(self.size,attr['npool']) + ('' if attr['npool']==1 else 's'))
+        if self.size == 1:
+          print('Serial execution')
+        else:
+          print('Parallel execution on %d processors and %d pool'%(self.size,attr['npool']) + ('' if attr['npool']==1 else 's'))
 
     # Do memory checks
-    if self.rank == 0:
+    if not restart and self.rank == 0:
       gbyte = self.memory_check()
       print('Estimated maximum array size: %.2f GBytes\n' %(gbyte))
 
@@ -149,6 +150,57 @@ class PAOFLOW:
 
 
 
+  def restart_dump ( self, fname_prefix='paoflow_dump' ):
+    '''
+      Saves the necessary information to restart a PAOFLOW run from any step in calculation.
+
+      Arguments:
+          fname_prefix (str): Prefix of the filenames which will be written. Files are written to the directory housing the python script which instantiates PAOFLOW, unless otherwise specified in this argument.
+
+      Returns:
+          None
+    '''
+    from pickle import dump,HIGHEST_PROTOCOL
+
+    fname = fname_prefix + '_%d'%self.rank + '.json'
+
+    arry,attr = self.data_controller.data_dicts()
+    with open(fname, 'wb') as f:
+      dump([arry,attr], f, HIGHEST_PROTOCOL)
+
+
+
+  def restart_load ( self, fname_prefix ):
+    '''
+      Loads the previously dumped save files and populates the DataController with said data.
+
+      Arguments:
+          fname_prefix (str): Prefix of the filenames which will be written. Files are written to the directory housing the python script which instantiates PAOFLOW, unless otherwise specified in this argument.
+
+      Returns:
+          None
+    '''
+    from os.path import exists
+    from pickle import load
+
+    fname = fname_prefix + '_%d'%self.rank + '.json'
+    if not exists(fname):
+      print('Restart file named %s does not exist.'%fname)
+      raise OSError('File: %s not found.'%fname)
+
+    arry,attr = None,None
+    with open(fname, 'rb') as f:
+      arry,attr = load(f)
+
+    if self.size != attr['mpisize']:
+      print('Restarted runs must use the same number of cores as the original run.')
+      raise ValueError('Number of processors does not match that of the previous run.')
+
+    self.data_controller.data_arrays = arry
+    self.data_controller.data_attributes = attr
+
+
+
   def memory_check ( self ):
     '''
     Estimate PAOFLOW's memory requirements with a "fudge factor" defined above
@@ -233,13 +285,12 @@ class PAOFLOW:
 
 
 
-  def pao_hamiltonian ( self, non_ortho=False, shift_type=1, write_binary=False, expand_wedge=True, symmetrize=False, thresh=1.e-6, max_iter=16 ):
+  def pao_hamiltonian ( self, shift_type=1, write_binary=False, expand_wedge=True, symmetrize=False, thresh=1.e-6, max_iter=16 ):
     '''
     Construct the Tight Binding Hamiltonian
     Yields 'HRs', 'Hks' and 'kq_wght'
 
     Arguments:
-        non_ortho (bool): If True the Hamiltonian will be Orthogonalized after construction
         shift_type (int): Shift type [ 0-(PRB 2016), 1-(PRB 2013), 2-No Shift ] 
 
     Returns:
@@ -252,7 +303,6 @@ class PAOFLOW:
     # Data Attributes and Arrays
     arrays,attr = self.data_controller.data_dicts()
 
-    if 'non_ortho' not in attr: attr['non_ortho'] = non_ortho
     if 'shift_type' not in attr: attr['shift_type'] = shift_type
     if 'write_binary' not in attr: attr['write_binary'] = write_binary
 
@@ -1040,7 +1090,7 @@ mo    '''
     attr['eminH'] = emin
     attr['emaxH'] = emax
 
-    if 'a_tensor' is not None: arrays['a_tensor'] = np.array(a_tensor)
+    if a_tensor is not None: arrays['a_tensor'] = np.array(a_tensor)
     if 'fermi_up' not in attr: attr['fermi_up'] = fermi_up
     if 'fermi_dw' not in attr: attr['fermi_dw'] = fermi_dw
 
